# Swer
## Introduction
Swer is working title for programming language designed for code-golf challenges. It's source code is compiled into C++14 compliant source code. Thus, swer could be thought of as JIT compiled language.

More information is provided in Text/prace/prace.pdf in czech language.

---
## Compilation
### Requirements
* flex ; (Lexer generator)
* bison ; (Parser generator)
* C++11 compliant compiler

### Windows
**Visual Studio 2015** works (2013 might work too).  
Make sure you have bison and flex utilities in `$PATH`, or change *Custom build tool* in visual studio for `src/swer.l` and `src/swer.y` (Solution explorer -> Properties).  
Hit build in Debug configuration (Release might not work yet).
It will create the file `.\Swer\Debug\Swer.exe`.

### Unix
Compiling on unix has additional requirements

- GNU utilities (make, sed)

In root directory of the project, execute following command:  
`$ make`  
It will create the file `Swer\swer`.

---
## Usage
`swer [options] file`

Will try to compile file (supposing Swer syntax) into C++ source code and writes it to std-out, unless `-o`, `-c` or `-C` commands are specified.

#### Options
```
 -h [--help]                    Displays this help information.
 -c [--compiler] [g++|clang|cl] Redirects the output to compiler and invokes it.
 -C [--command] <command>       Redirects output to <command> and invokes it.
 -o [--output] <file>           Instead of writing to std-out, writes to file
```

`-C` and `-c` command can't be mixed. If `-o` is used with `-c`, then the `-o` option is passed to the compiler accordingly. `-c` is functionally equivalent to `-C` like this:
```
-c g++   <=> -C "g++ -std=c++14 -x c++ - "
-c clang <=> -C "clang++ -std=c++14 -x c++ - "
-c cl    <=> -C "cl /EHsc /TP /Fea.exe tmpfile" 
```
Output executable from `cl` (Microsoft C++ compiler) is a.exe, unless overridden by `-o` option. `tmpfile` is created using [`std::tmpnam`](http://en.cppreference.com/w/cpp/io/c/tmpnam). `tmpfile` is not deleted, nor is `tmpfile.obj`, which is object file produced by `cl`.

Command:  
`./swer file`  
will compile `file` to C++ source code and write it to standard output.

---
## Quick Syntax Reference
The syntax is quite same as in any other C-like structural language.  
For better understanding of following sections, please consider this

* language is case-**sensitive**, that means `Foo` and `foo` are different identifiers.
* white spaces (space, newline, tab) are optional, unless it would merge two words together (in `int foo`, space is mandatory, but can be replaced by tab or newline).
* **`keyword`** is a keyword, and must appear in exact position and is not to be changed.
* `CamelCase` is identifier, which must suffice following regular expression `[a-zA-Z_][a-zA-Z_0-9]*`, that is a letter or underscore followed by zero or more letters, underscores or numbers.
* `text1`,... means that `text` can be repeated multiple times using comma (,) as delimiter.
* *italic text* signalizes, that the part is optional.
* text_with_underscores is reference for a section (variable_declaration is described in **Variable** section).

Program is a sequence of declarations (potentially empty), followed by a sequence of statements (potentially empty).  
Only single line comments exists in Swer. Anything from `//` up to the end of line is considered a comment and is ignored by the compiler.

### Declarations
Declaration can be either class declaration or function declaration.
#### Class
*`{` `T1`, ...`}`* `ClassName`  
`{`  
*sequence_of_declarations_of_classes_variables_or_functions*  
`}`  
Defines class with name `ClassName` and optional type arguments `T1`, ...
#### Function
`ReturnType` `FunctionName` `(`*`Type1`* *`Param1`* *`=`* *expression*, ...`)`   
`{`  
*sequence_of_statements*  
`}`  
Defines a function `FunctionName` returning `ReturnType` and having `Type1 param1` arguments.

#### Variable
`Type` `var` *`=`* *expression*`;`  
Declares a variable `var` of type `Type` and optionally assigns a value to it.

### Statements
#### Expression
expression `;`  
Evaluates an expression.

#### Compound
`{`  
*sequence_of_statements*  
`}`  
Used to group statements together.

#### Conditions
**`if`** `(` condition `)` statement1 _**`el`** statement2_  
If `condition` is true, `statement1` is executed and optionally if `condition` is false, `statement2` is executed.  

This suffers from [dangling-else issue](https://en.wikipedia.org/wiki/Dangling_else) (as many others C like languages) and is resolved using same method, that is, pairing else with closest if.

See `examples/` directory in repository.

#### Switch
**`sw`** `(` expression `)`  
`{`  
`:`expression1`:` *sequence_of_statements1*  
...  
*`:` __`el`__ `:`  sequence_of_statements*  
`}`  
Switch statement, evaluates `expression` and based on its value comparing to `expressionN` it will execute `sequence_of_statementsN`. If no branch equals `expression`, optional `el` branch is executed.  

In Swer, there is no implicit fall trough switch branches, unless sequence of statements is empty. To introduce fall through, last statement has to be `cnt;`, explicit exit from switch via `brk;` works as well.

See `Examples/` directory in repository.

#### Iteration
* `@` *`Identifier`* `(`expression1`..`expression2 `)` statement [1]
* `@` *`Identifier`* `(`expression`)` statement [2]
* `@` `(` expression`;` `)` statement [3]
* `@` `(` expression1 `;` expression2 `;` expression3 `)` statement [4]
* `@` `(` variable_declaration `;` expression1 `;` expression2 `)` statement [5]

[1]: For cycle from `expression1` to `expression2` inclusive with a step of +-1  
[2]: For range cycle on values of `expression`.  
[3]: While loop, until `expression` is true.  
[4]: C-like for loop, where `expression1` is initialization, `expression2` is condition and `expression3` is step.  
\[5\]: C-like for loop, with new variable declaration (with mandatory initialization)  

In \[1\] and \[2\], `Identifier` is optional, if not specified, compiler will generate identifier using following rules:
* If this is first for cycle with implicit identifier, with respect to scope, `Identifier` is `i`
* If this is nested for cycle (nested in exactly one for cycle with implicit identifier), `Identifier` is `j`
* If this is nested for cycle (nested in exactly two for cycles with implicit identifier), `Identifier` is `k`
* If this is nested for cycle within 3 and more for cycles with implicit identifier, compilation error occurs.
* If `Identifier` is already taken (user explicitly declared variable with such name), compilation error occurs.

See `Examples/` directory in repository.

#### Declaration
variable_declaration `;`

Declares new variable with optional value (see **Variable**). Furthermore, you can assign a value to it, by calling it's type constructor and passing arguments to it with following syntax  
`Type` `variable` `{` *`arg1`, ...* `}`

#### Keywords
**`brk`** `;`  
**`cnt`** `;`  
`->` *expression* `;`  
Will contain special `throw`, which will either return from the function, or `throw` an uncatchable exception (since swer does not have `try`-`catch` mechanics).

**`brk`** is equivalent to `break`, **`cnt`** is equivalent to `continue`, `->` is equivalent to `return` in C-like languages. (See more on **Switch** and **Iteration**).

#### Expressions
Expressions work pretty much the same as in C with few exceptions, which will be discussed later on.

* literal (number, char, string) [1] 
* identifier [2]
* expression`(` *`arg1`*,...`)`  [3]
* expression`[`expression`]`  [4]
* expression1 operator expression2  [5]

[1]: decimal literal, character literal or string literal (See [] for more)  
[2]: variable, class name  
[3]: Function call  
[4]: Array access  
[5]: Operator can be  

* `+` `-` `*` `/` as in arithmetics 
* `&` `|` `~` for bit-wise `and` `or` `not`
* `&&` `||` `!` for logical `and` `or` `not` (highly likely, that logical and bit-wise and/or will have it's operators swapped)
* `^` as exponentiation
* `<<` `>>` as bit-wise shifts
* All of previous combined with `=` (like `+=`, `&=`), except unary operators (`~`, `!`)
* `++` `--` as increment and decrement (both pre- and post- versions)
* `&` `*` address-of, dereference (see pointers)
* `.` `->` as class member access
* `<` `>` `<=` `>=` `==` `!= comparisons

See [] for more information about precedence and associativity.  

---
### Types
Swer has no built in types, but it's standard library has support for integers, strings, arrays and much more. (See **STL** section)  
Also, there is a special type `$`, which is any type (determined at compile time, same as C++'s `auto`). It can only be used in variable declaration if it contains initialization, but function return type or even input arguments can be declared as `$` (See [] for more).

One can add sequence of following qualifiers to a type name:
* \# ; stands for pointer, equivalent to `*` in C++
* ~ ; stands for reference, equivalent to `#` in C++.

Beware, that qualifier is part of type, not variable (as is in C++), `int #i` is equivalent to `int# i` and declares a variable `i` of type pointer to `int`. `int i, #j` is invalid syntax and will result in compile error.

---
### Swer STL
Current standard library provides facilities for working with strings, arrays and integers. In order to use STL, you have to instruct the compiler to find `SwerSTL/src/swer_stl`.

Detailed documentation of STL will be added someday, since then, use this quick reference. Also, everything is still under high development and keywords, names or whatever might change for no specific reason.

Note, that containers (`string`, `vector`) most common function usually have three forms: full-name (`reverse()`), shortened name to two letters (`rv()`) and one letter getter (`r`).

##### `integer` (or just `I`)
Is plain number, represented with [`long long`](http://en.cppreference.com/w/cpp/language/types), thus it is at least 64-bit large, singed integer.

Literal of this type is just plain number (`0`, `-5`, `180`).

##### `floating` (or just `F`)
Is floating point number, represented with [`double`](http://en.cppreference.com/w/cpp/language/types), thus it should have properties of IEEE-754 64 bit floating point type.

Literal of this type is floating point number (`1.0`, `0.`, `.5`, `1e9`). 

##### `string` (or just `S`)
Is string container, implemented using [`std::string`](http://en.cppreference.com/w/cpp/string/basic_string) and has pretty much same API with few additions.

- `size()` or `sz()` returns size. Can also be retrieved using `s` member (not function, just member).
- `operator[](n)` will access `n`-th element in the vector (zero-based) using `circular_indexer` (see below for details). 
- `reverse()` or `rv()` returns new reversed string. Also usable using `r` member.
- `split(delimiters = ",")` or `sp(delimiters = ",")`, also accessible using `v` as *vectorize*, splits the string into substring delimited by argument.
- Is implicitly convertible to `integer`.

Literal of this type is any string literal (`"abc"`)


##### `vector{T}` (or just `V{T}`)
Is generic container for storing elements of type `T`. It is implemented using [`std::deque`](http://en.cppreference.com/w/cpp/container/deque) and it has pretty much same API with few additions.

- `size()` or `sz()` returns size. Can also be retrieved using `s` member (not function, just member).
- `operator+` can merge two `vector`s. 
- `operator+=(T)` can add element to the `vector`. Syntax is `v += 1;` for appending, and `1 += v` for prepending.
- `operator+=(vector<T>)` can add multiple elements to the `vector`. Syntax is `v += [1,2,3];` for appending, and `[1,2,3] += v` for prepending. Note, that argument has to be *rvalue*, thus `v += v;` does not work and will result in compile error.
- `operator[](n)` will access `n`-th element in the vector (zero-based) using `circular_indexer` (see below for details).
- `sort()` or `sr()` returns new sorted array. Also accessible using `o` as *ordered*.
- `reverse()` or `rv()` returns new reversed array. Also usable using `r` member.


Literal of this type is created using brackets (`[1,2,3]` is `vector{integer}`, `["a", "b", "c"]` is `vector{string}`). Type of `vector` is deduced using it's first argument. If rest arguments are not of same type (or implicitely convertible), it will result to compilation error.  

##### Output mechanism
Swer allows it's user to output anything to std-out, not anywhere else (for now) and uses [`std::cout`](http://en.cppreference.com/w/cpp/io/cout) for that. Printing is done using **`print(...)`** function (or just **`p(...)`**), which decays to `std::cout`. That means, it does not support printing custom types. `print` also prints newline after it's call, which can't be suppressed now.

##### `circular_indexer`
While not actually object accessed explicitly from STL, `string` and `vector` use this utility while indexing (using `operator[](n)`). It's behavior is as follows, supposing first element has index `0`.

- if size of container is `0`, throws an exception.
- if `n` is positive, returns `n % size`-th element of the container.
- if `n` is negative, returns `-n % size`-th element of the container from the end.

This simulates circular behavior of containers.

##### Command line arguments (CLR)
Swer provides it's user a way to access arguments passed to executable. Command line arguments are stored in a variable declared as follows

    vector{string} _;

That implies, that one can not declare any variable named `_` at swer's global scope.

Also `_` does not contain first (`0`) argument as passed to C++ -- Name of file being run.

---
## Examples
Are located in `Examples\` folder in repository.

---
## Remarks
Not everything is implemented now, current version (as of commit `4f5d4af39ae224692567115ae6e3e62306357851`) parses everything except lambdas (which are not yet described), but only outputs sequence of statements, but not class or function declarations. Also, error messages are not quite clear. Furthermore, standard library is not present yet.
More information will come some day.