﻿#pragma once
#include <string>
#include <memory>
#include <vector>

namespace swer
{
    struct base_declaration;
    class class_declaration;
    class function_declaration;
    class variable_declaration;

    using base_declaration_ptr = std::shared_ptr<base_declaration>;
    using base_declaration_seq = std::vector<base_declaration_ptr>;
    using class_declarations = std::vector<const class_declaration*>;

    /**
        Represents all declarations in Swer. class, function, variable declarations classes derive from this.
    */
    struct base_declaration
    {
        /**
            Returns textual representation of given declaration. Derived classes shall override this
            method to provide implementation.
        */
        virtual std::string as_string() const = 0;
        virtual ~base_declaration() = default;
        
        /**
            Casts base_declaration to class_declaration.
        */
        static std::shared_ptr<class_declaration> as_class(const base_declaration_ptr& ptr);

        /**
            Checks, whether ptr points to class_declaration object.
        */
        static bool is_class(const base_declaration_ptr& ptr) { return as_class(ptr) != nullptr; }

        /**
            Casts base_declaration to function_declaration.
        */
        static std::shared_ptr<function_declaration> as_function(const base_declaration_ptr& ptr);

        /**
            Checks, whether ptr points to function_declaration object.
        */
        static bool is_function(const base_declaration_ptr& ptr) { return as_function(ptr) != nullptr; }

        /**
            Casts base_declaration to variable_declaration.
        */
        static std::shared_ptr<variable_declaration> as_variable(const base_declaration_ptr& ptr);

        /**
            Checks, whether ptr points to variable_declaration object.
        */
        static bool is_variable(const base_declaration_ptr& ptr) { return as_variable(ptr) != nullptr;  }
    };
}
