﻿#pragma once
#include <memory>
#include <vector>

namespace swer
{
    /**
        Represents a statement in Swer. All statement classes derive from this struct.
    */
    struct base_statement
    {
        /**
            Returns string representation of statement. Is overriden in derived classes.
        */
        virtual std::string as_string() const = 0;
        virtual ~base_statement() = default;
    };

    using base_statement_ptr = std::shared_ptr<base_statement>;
    using base_statement_seq = std::vector<base_statement_ptr>;

    /**
        Keywords used in keyword statements.
    */
    enum class keywords
    {
        RETURN, // ->
        BREAK, // brk
        CONTINUE, // cnt
    };
}

// Was fixed in some version of g++
// G++ does not partialy specialize std::hash for plain enum types.
#ifdef __GNUC__
namespace std
{
    template<>
    struct hash<swer::keywords>
    {
        size_t operator()(const swer::keywords& t) const
        {
            return hash<size_t>()(static_cast<size_t>(t));
        }
    };
}
#endif
