#pragma once
#include <memory>

#include "base_statement.hpp"
#include "base_expression.hpp"
#include "utilities.hpp"
#include "keyword_statement.hpp"

namespace swer
{
    /**
    Represents case statement in Swer.
    */
    class case_statement : public base_statement
    {
    public:
        case_statement(base_expression_ptr&& case_, base_statement_seq statements)
            : m_case(std::move(case_)), m_statements(std::move(statements))
        {
        }

        explicit case_statement(base_statement_seq statements)
            : case_statement(nullptr, std::move(statements))
        {
        }

        explicit case_statement(base_expression_ptr&& case_)
            : case_statement(std::move(case_), base_statement_seq { })
        {
        }

        case_statement()
            : case_statement(nullptr, base_statement_seq { })
        {
        }

        std::string as_string() const override
        {
            std::string str;
            str += concat(m_case != nullptr ? concat("case ", m_case->as_string()) : "default", ":\n");
            if (m_statements.empty())
                return std::move(str);

            if (ends_with_continue())
                str += string_join(m_statements.begin(), std::prev(m_statements.end()), "", as_string_decay<base_statement_ptr>);
            else
                str += concat(string_join(m_statements, "", as_string_decay<base_statement_ptr>), "break;\n");
            return std::move(str);
        }

    private:
        /**
        Checks whether last statement in this case is cnt;
        */
        bool ends_with_continue() const
        {
            auto keyword_stmt = std::dynamic_pointer_cast<keyword_statement>(m_statements.back());
            if (keyword_stmt != nullptr && keyword_stmt->get_keyword() == keywords::CONTINUE)
                return true;
            return false;
        }

        base_expression_ptr m_case;
        base_statement_seq m_statements;
    };
}
