#pragma once
#include "base_expression.hpp"
#include "variable_declaration.hpp"

namespace swer
{
    /**
    Represents a condition in Swer. That is used in if and for statements and allows user to either declare
    a variable or use just expression.
    */
    class condition : public base_expression
    {
    public:
        explicit condition(base_expression_ptr&& expression) : m_expression(std::move(expression))
        {
        }

        explicit condition(variable_declaration_ptr&& variable_decl) : m_variable_decl(std::move(variable_decl))
        {
        }

        std::string as_string() const override
        {
            return m_expression != nullptr ? m_expression->as_string() : m_variable_decl->as_string();
        }

    private:
        // ToDo: variant
        base_expression_ptr m_expression;
        variable_declaration_ptr m_variable_decl;
    };
}
