#pragma once
#include <string>

#include "base_expression.hpp"
#include "utilities.hpp"
#include "compiler.hpp"
#include "iteration_statement.hpp"

namespace swer
{
    /**
    Represents for cycle with fixed from, to points.
    */
    class for_from_to_statement : public iteration_statement
    {
    public:
        for_from_to_statement(std::string&& identifier, base_expression_ptr&& from, base_expression_ptr&& to, base_statement_ptr&& body, compiler* comp)
            : iteration_statement(std::move(body))
            , m_identifier(std::move(identifier)), m_from(std::move(from)), m_to(std::move(to))
            , m_from_var(comp->get_tmp_variable_identifier()), m_to_var(comp->get_tmp_variable_identifier())
        {
        }

        for_from_to_statement(base_expression_ptr&& from, base_expression_ptr&& to, base_statement_ptr&& body, compiler* comp)
            : for_from_to_statement(comp->get_iteration_identifier(), std::move(from), std::move(to), std::move(body), comp)
        {
        }

        std::string as_string() const override
        {
            return concat(
                "{\n",
                "const auto& ", m_from_var, " = ", m_from->as_string(), ";\n",
                "const auto& ", m_to_var, " = ", m_to->as_string(), ";\n",
                "for (auto ", m_identifier, " = ", m_from_var, "; ",
                m_from_var, " < ", m_to_var, " ? ", m_identifier, " <= ", m_to_var, " : ", m_identifier, " >= ", m_to_var, "; ",
                m_from_var, " < ", m_to_var, " ? ", "++", m_identifier, " : ", "--", m_identifier, ")\n",
                m_body->as_string(),
                "}\n"
            );
        }

    private:
        std::string m_identifier;
        base_expression_ptr m_from;
        base_expression_ptr m_to;

        std::string m_from_var, m_to_var;
    };
}
