#pragma once
#include <string>

#include "base_expression.hpp"
#include "utilities.hpp"
#include "compiler.hpp"
#include "iteration_statement.hpp"

namespace swer
{
    /**
    Represents for-each cycle which iterates over a container.
    */
    class for_range_statement : public iteration_statement
    {
    public:
        for_range_statement(std::string identifier, base_expression_ptr&& range, base_statement_ptr&& body)
            : iteration_statement(std::move(body))
            , m_identifier(std::move(identifier)), m_range(std::move(range))
        {
        }

        for_range_statement(base_expression_ptr&& range, base_statement_ptr&& body, compiler* comp)
            : for_range_statement(comp->get_iteration_identifier(), std::move(range), std::move(body))
        {
        }

        std::string as_string() const override
        {
            return concat(
                "for (auto& ", m_identifier, " : ", m_range->as_string(), ")\n",
                m_body->as_string()
            );
        }

    private:
        std::string m_identifier;
        base_expression_ptr m_range;
    };
}
