#pragma once
#include "base_expression.hpp"
#include "variable_declaration.hpp"
#include "base_statement.hpp"
#include "iteration_statement.hpp"

namespace swer
{
    /**
    Represents a for statement in Swer, which translates to for cycle in C++.
    */
    class for_statement : public iteration_statement
    {
    public:
        for_statement(variable_declaration_ptr&& variable, base_expression_ptr&& condition, base_expression_ptr&& step, base_statement_ptr&& body)
            : iteration_statement(std::move(body))
            , m_variable(std::move(variable)), m_condition(std::move(condition)), m_step(std::move(step))
        {
        }

        for_statement(base_expression_ptr&& initialization, base_expression_ptr&& condition, base_expression_ptr&& step, base_statement_ptr&& body)
            : iteration_statement(std::move(body))
            , m_initialization(std::move(initialization)), m_condition(std::move(condition)), m_step(std::move(step))
        {
        }

        std::string as_string() const override
        {
            return concat(
                "for (", m_variable != nullptr ? m_variable->as_string() : m_initialization->as_string(), "; ",
                m_condition->as_string(), "; ",
                m_step->as_string(), ")\n",
                m_body->as_string()
            );
        }

    private:
        variable_declaration_ptr m_variable;
        base_expression_ptr m_initialization;
        base_expression_ptr m_condition;
        base_expression_ptr m_step;
    };
}
