﻿#pragma once
#include <utility>
#include <memory>
#include <string>

#include "type.hpp"
#include "variable_declaration.hpp"
#include "expression.hpp"
#include "base_statement.hpp"
#include "function_declaration.hpp"
#include "class_declaration.hpp"
#include "compiler.hpp"
#include "numerics.hpp"
#include "utilities.hpp"

namespace swer
{
    constexpr int CODE_EXIT_SUCCESS = 0;
    constexpr int CODE_EXIT_FAILURE = 1;

    class compiler;
    class simple_logger;

    extern ::std::unique_ptr<simple_logger> global_logger;

    // ToDo: variant
    struct swer_lval
    {
        ::std::string string;
        integer number_i;
        floating number_f;

        ::std::size_t index; // for indexing tokens (actually yytokentype)

        identifier_seq identifier_seq;
        
        type_qualifier_ptr type_qualifier;
        type_qualifier_seq type_qualifiers;
        type_specifier_ptr type;
        type_specifier_seq type_seq;
        
        identifier_initialization identifier_init;
        identifier_initialization_seq identifier_init_seq;
        base_declaration_ptr declaration;
        base_declaration_seq declaration_seq;

        formal_parameter_ptr formal_params;
        formal_parameter_seq formal_params_seq;

        base_expression_ptr expression;
        base_statement_ptr statement;
        base_statement_seq statement_seq;
    };
}

#define YY_DECL	int yylex(swer::swer_lval* lval, unsigned* line, swer::compiler* comp)
YY_DECL;
#define YYSTYPE swer::swer_lval
int yyparse(swer::compiler* comp);
extern FILE* yyin;
