#pragma once
#include <algorithm>
#include <string>
#include <memory>

#include "base_statement.hpp"
#include "utilities.hpp"
#include "condition.hpp"

namespace swer
{
    /**
    Represents conditional if statement with optional else branch.
    */
    class if_statement : public base_statement
    {
        using condition_ptr = std::shared_ptr<condition>;
    public:
        if_statement(condition_ptr&& condition, base_statement_ptr if_branch, base_statement_ptr else_branch = nullptr)
            : m_condition(std::move(condition))
            , m_if_branch(std::move(if_branch)), m_else_branch(std::move(else_branch))
        {
        }

        std::string as_string() const override
        {
            std::string str;
            str += concat("if (", m_condition->as_string(), ")\n", m_if_branch->as_string());
            if (m_else_branch != nullptr)
                str += concat("else\n", m_else_branch->as_string());

            return std::move(str);
        }

    private:
        condition_ptr m_condition;
        base_statement_ptr m_if_branch;
        base_statement_ptr m_else_branch;
    };

}
