#pragma once
#include <unordered_map>
#include "utilities.hpp"

#include "base_expression.hpp"
#include "base_statement.hpp"

namespace swer
{
    /**
    Represents a statement consisting of keyword and optional expression (brk; cnt; ->expr;)
    */
    class keyword_statement : public base_statement
    {
    public:
        explicit keyword_statement(keywords keyword, base_expression_ptr&& expression = nullptr)
            : m_keyword(keyword), m_expression(std::move(expression))
        {
        }

        std::string as_string() const override
        {
            return concat(
                keyword(m_keyword), " ", m_expression != nullptr ? m_expression->as_string() : "", ";\n"
            );
        }

        /**
        Returns keyword of this statement.
        */
        keywords get_keyword() const
        {
            return m_keyword;
        }

    private:
        /**
        Returns C++ keyword for given enum item.
        */
        static std::string keyword(keywords key)
        {
            static std::unordered_map<keywords, std::string> mapping {
                { keywords::RETURN, "return" },
                { keywords::BREAK, "break" },
                { keywords::CONTINUE, "continue" },
            };

            try
            {
                return mapping.at(key);
            }
            catch (std::out_of_range&)
            {
                throw std::runtime_error("Added new keyword without text representation.");
            }
        }

        keywords m_keyword;
        base_expression_ptr m_expression;
    };
}
