#pragma once
#include "base_statement.hpp"
#include "base_expression.hpp"
#include "utilities.hpp"

namespace swer
{
    /**
    Represents a lambda expression in swer. It has identifiers (parameters) without types (uses auto deduction in lambda)
    and statement forming a body of lambda.
    */
    class lambda : public base_expression
    {
    public:
        explicit lambda(identifier_seq&& identifiers, base_statement_ptr&& statement)
            : m_identifiers(std::move(identifiers)), m_statement(std::move(statement))
        {
        }

        std::string as_string() const override
        {
            return concat(
                "[&](",
                string_join(m_identifiers, ", ", [](const std::string& item) { return concat("const auto& ", item); }),
                ")",
                m_statement->as_string()
            );
        }

    private:
        identifier_seq m_identifiers;
        base_statement_ptr m_statement;
    };
}
