#pragma once
#include <string>
#include <sstream>
#include <iomanip>

#include "base_expression.hpp"
#include "utilities.hpp"
#include "numerics.hpp"

namespace swer
{
    /**
    Represents a literal of type T in Swer. Arrays are represented by array class and not literal<T[]>.
    */
    template<typename T>
    class literal : public base_expression
    {
    public:
        explicit literal(T&& lit) : m_literal(std::forward<T>(lit))
        {
        }

        std::string as_string() const override
        {
            return to_string_hlp(m_literal);
        }

    private:
        /**
        Helper functions with converts arguments based on type T.
        Could be partialy specialized for each T type, but it would
        make the code much longer.
        */
        static std::string to_string_hlp(const std::string& str)
        {
            return concat("String(\"", str, "\")");
        }

        static std::string to_string_hlp(char c)
        {
            return concat("'", std::string(1, c), "'");
        }

        static std::string to_string_hlp(integer i)
        {
            return std::to_string(i) + "ll";
        }

        static std::string to_string_hlp(floating f)
        {
            std::stringstream ss;
            ss << std::setprecision(std::numeric_limits<float>::digits10 + 1) << f;
            return ss.str();
        }

        template<typename U>
        static std::string to_string_hlp(const U& u)
        {
            return std::to_string(u);
        }

        T m_literal;
    };
}
