#pragma once
#include "base_expression.hpp"
#include "utilities.hpp"

namespace swer
{
    /**
    Represents an operator object, which is primarily used by passing them to functions accepting binary lambda.
    */
    class operator_object : public base_expression
    {
    public:
        /**
        Binary operator type
        */
        enum operator_type
        {
            PLUS, // a + b
            MINUS, // a - b
            MULTI, // a * b
            DIVIDE, // a / b
            MODULO, // a % b
            EXPONENT, // a ^ b
        };
    private:
        static std::string operator_symbol(operator_type type);

    public:
        explicit operator_object(operator_type type) : m_type(type)
        {
        }

        std::string as_string() const override
        {
            return concat("[](auto lhs, auto rhs){ return lhs ", operator_symbol(m_type), " rhs; }");
        }

    private:
        operator_type m_type;
    };
}

// Was fixed in some version of g++
// G++ did not provide partial specializations of std::hash for enum types.
#ifdef __GNUC__
namespace std
{
    template<>
    struct hash<swer::operator_object::operator_type>
    {
        size_t operator()(const swer::operator_object::operator_type& t) const noexcept
        {
            return hash<size_t>()(static_cast<size_t>(t));
        }
    };
}
#endif
