#pragma once
#include "base_expression.hpp"
#include "base_statement.hpp"
#include "utilities.hpp"

namespace swer
{
    /**
    Represents switch statement in Swer.
    */
    class switch_statement : public base_statement
    {
    public:
        switch_statement(base_expression_ptr&& condition, base_statement_seq&& statements)
            : m_condition(std::move(condition)), m_statements(std::move(statements))
        {
        }

        std::string as_string() const override
        {
            return concat(
                "switch (", m_condition->as_string(), "){\n",
                string_join(m_statements, "\n", as_string_decay<base_statement_ptr>),
                "}\n"
            );
        }

    private:
        base_expression_ptr m_condition;
        base_statement_seq m_statements;
    };
}
