﻿#pragma once
#include <iostream>
#include <sstream>
#include "string.hpp"
#include "numerics.hpp"
#include "vector.hpp"
#include "map.hpp"

namespace swerstl
{
    /**
        Reads all of standard input to a String.
    */
    inline String read_to_end()
    {
        std::ostringstream sstream;
        sstream << std::cin.rdbuf();
        auto str = sstream.str();
        str.pop_back(); // newline
        return String(str);
    }

    /**
        Reads from standard input until new line character is encountered and returns that as String.
    */
    inline String read_line()
    {
        String str;
        getline(std::cin, str);
        return str;
    }

    /**
        Reads next element of type T from standard input. If element cound not be read, returns defaultly constructed T.
    */
    template<typename T>
    T read()
    {
        T t;
        if (std::cin >> t)
            return t;
        return T{};
    }

    inline String re() { return read_to_end(); }
    inline String rl() { return read_line(); }
    inline String rs() { return read<String>(); }
    inline Integer ri() { return read<Integer>(); }
    inline Floating rf() { return read<Floating>(); }
    inline Character rc() { return read<Character>(); }
    
    template<typename T>
    Vector<T> rv(Vector<T>& v) { return v = read<Vector<T>>(); }
    template<typename K, typename V>
    Map<K, V> rm(Map<K, V>& v) { return v = read<Map<K, V>>(); }
}
