﻿#pragma once
#include <utility>

namespace swerstl
{
    /**
        Wrapper around std::pair used in Map::iterator.
        Provides similar functionality as std::pair does, but has shortened retrieval of elements.
    */
    template<typename First, typename Second>
    class Pair
    {
    public:
        using underlaying_container = std::pair<First, Second>;
        using first_type = First;
        using second_type = Second;

        /**
            Constructs new pair by forwarding arguments.
        */
        Pair(first_type&& first, second_type&& second)
            : m_pair(std::forward<first_type>(first), std::forward<second_type>(second))
            , f(m_pair.first), s(m_pair.second)
        {
        }

        // ReSharper disable once CppNonExplicitConvertingConstructor
        Pair(underlaying_container&& pair) : m_pair(std::move(pair))
        {
        }

    private:
        underlaying_container m_pair;

    public:
        first_type& f; // 'f'irst
        second_type& s; // 's'econd
    };
}
