﻿#pragma once
#include "numerics.hpp"

namespace swerstl
{
    namespace detail
    {
        /**
         * Returns an element from Container at position index, with following modifications.
         *     if index >= 0, returns index % container.size() element from the beginning.
         *     if index < 0, returns -index % container.size() from the end.
         */
        template<typename Container>
        typename Container::reference circular_indexer(Container& container, Integer index)
        {
            if (!container.size())
                throw std::out_of_range("Attempting to index an empty container.");

            if (index < 0)
                return container.at(container.size() - 1 - (-index - 1) % container.size());
            return container.at(index % container.size());
        }

        template<typename Container>
        typename Container::const_reference circular_indexer(const Container& container, Integer index)
        {
            if (!container.size())
                throw std::out_of_range("Attempting to index an empty container.");

            if (index < 0)
                return container.at(container.size() - 1 - (-index - 1) % container.size());
            return container.at(index % container.size());
        }
    }
}
