﻿#pragma once
#include <functional>

namespace swerstl
{
    namespace detail
    {
        /**
         * Implementation of getter mechanism.
         * Almost equivalent to getters in Java/C#, but has issues in template argument deduction
         *     (i.e. getter<T> instead of T is deduced)
         */
        template<typename T>
        class getter
        {
        public:
            /**
             * Constructs an empty getter, which, when called, will throw std::bad_function_call.
             */
            getter()
            {
            }

            /**
             * Constructs a getter by providing a reference to a function.
             */
            explicit getter(const std::function<T(void)>& fnc) : m_getter(fnc)
            {
            }

            /**
             * Implicit conversion operator to return desired value, by calling stored function.
             */
            operator T() const
            {
                return m_getter();
            }

        private:
            /// Stored function, which, upon calling, returns desired value.
            std::function<T(void)> m_getter;
        };
    }
}
