args = commandArgs(trailingOnly=TRUE)
if (length(args) == 0)
{
    stop("Missing argument")
}

library(ggplot2)
theme_set(theme_bw())
data <- read.csv(args[1])

startup <- ggplot(data, aes(type, startup)) +
    geom_boxplot(outlier.size=0.4) +
    labs(
        title = "Startup time",
        x = "Agent used",
        y = "Time in ms"
    ) + 
    scale_x_discrete(
        limits = c("none", "events", "bci", "bci.all", "bciinfo", "bciinfo.all"),
        labels = c("None", "Events", "BCI", "BCI All", "BCI Info", "BCI Info All"),
    ) +
    coord_flip() +
    facet_grid(benchmark ~ ., labeller = as_labeller(c(`MatrixMultiplication` = "MatMult", `VirtualCalls` = "VCalls", `Zip` = "Zip")))
    
execution <- ggplot(data, aes(type, startup + execution)) +
    geom_boxplot(outlier.size=0.4) +
    labs(
        title = "Total execution time",
        x = "Agent used",
        y = "Time in ms"
    ) + 
    scale_x_discrete(
        limits = c("none", "events", "bci", "bci.all", "bciinfo", "bciinfo.all"),
        labels = c("None", "Events", "BCI", "BCI All", "BCI Info", "BCI Info All"),
    ) +
    scale_y_continuous(
        breaks = scales::pretty_breaks(n = 10)
    ) +
    coord_flip() +
    facet_grid(benchmark ~ ., labeller = as_labeller(c(`MatrixMultiplication` = "MatMult", `VirtualCalls` = "VCalls", `Zip` = "Zip")))

events <- ggplot(aggregate(. ~ type+benchmark, data, sum), aes(type, events / execution)) +
    geom_bar(stat = "identity") +
    labs(
        title = "Ratio of events per time",
        x = "Agent used",
        y = "Average number of events per ms"
    ) + 
    scale_x_discrete(
        limits = c("none", "events", "bci", "bci.all", "bciinfo", "bciinfo.all"),
        labels = c("None", "Events", "BCI", "BCI All", "BCI Info", "BCI Info All"),
    ) +
    scale_y_log10(
        labels = scales::comma,
        breaks = c(1, 10, 100, 1000, 10000, 100000),
        expand = c(0, 0),
        limits = c(1, 150000)
    ) +
    coord_flip() +
    facet_grid(benchmark ~ ., labeller = as_labeller(c(`MatrixMultiplication` = "MatMult", `VirtualCalls` = "VCalls", `Zip` = "Zip")))

ggsave("../report/Startup.pdf", startup, height = 3)
ggsave("../report/Execution.pdf", execution, height = 3)
ggsave("../report/Events.pdf", events, height = 3)
