﻿#pragma once
#include <string>
#include <stdexcept>
#include <algorithm>

#include "bci_server.hpp"
#include "make_unique.hpp"

namespace runtime_checker
{
    class instrumentation_client
    {
    public:
        instrumentation_client(const std::string& class_name, const unsigned char* class_data, jint class_data_len)
            : class_name(class_name), class_data(class_data), class_data_len(class_data_len)
        {
        }

        std::vector<unsigned char> request_instrumentation(instrumentation_server& bci_server) const
        {
            return do_request_instrumentation(bci_server);
        }

    private:
        std::string class_name;
        const unsigned char* class_data;
        jint class_data_len;

        std::vector<unsigned char> do_request_instrumentation(instrumentation_server& bci_server) const
        {
            auto data = bci_server.perform_instrumentation(class_name, class_data, class_data_len, class_name == "java/lang/Object");
            auto array = std::vector<unsigned char>(data.size());
            std::copy(data.begin(), data.end(), array.data());
            return array;
        }
    };
} // namespace runtime_checker
