﻿#pragma once
#include <type_traits>
#include <memory>

namespace nonstd
{
    /**
      * C++14 std::make_unique implementation using SFINAE to distinguish between array and plain std::unique_ptr.
      */
    template<typename T, typename... Args>
    typename std::enable_if<!std::is_array<T>::value, std::unique_ptr<T>>::type make_unique(Args&&... args)
    {
        return std::unique_ptr<T>(new T(std::forward<Args>(args)...));
    }

    template<typename T>
    typename std::enable_if<std::is_array<T>::value, std::unique_ptr<T>>::type make_unique(std::size_t size)
    {
        return std::unique_ptr<T>(new typename std::remove_extent<T>::type[size]());
    }
} // namespace nonstd
