﻿#pragma once
#include "platform.hpp"

#include <cstdint>
#include <vector>
#include <string>
#include <memory>

#ifdef PLATFORM_WIN
  #include <winsock2.h>
#endif
#ifdef PLATFORM_UNIX
  #include <arpa/inet.h>
  #include <netinet/in.h>
#endif

namespace nonstd
{
    class connection
    {
        constexpr static const int TIMEOUT = 5 * 1000;
    public:
        using byte = char;

        connection(std::string address, int port, bool delayed_connect = false);
        ~connection();

        void connect();
        void disconnect();

        void send_message(const std::string& message) const
        {
            send(message.data(), message.size());
        }

        std::string recv_message(std::size_t size) const
        {
            std::vector<byte> data(size);
            recv(data.data(), data.size());
            return { data.begin(), data.end() };
        }

        void send_int32(std::int32_t num) const
        {
            std::int32_t netorder = htonl(num);
            send(reinterpret_cast<byte*>(&netorder), sizeof netorder);
        }

        std::int32_t recv_int32() const
        {
            std::vector<byte> data(sizeof(std::int32_t));
            recv(data.data(), data.size());
            std::int32_t num = *reinterpret_cast<std::int32_t*>(data.data());
            return ntohl(num);
        }

        void send_data(const unsigned char* data, std::size_t size) const
        {
            send(reinterpret_cast<const byte*>(data), size);
        }

        void send_data(const std::vector<byte>& data) const
        {
            send(data.data(), data.size());
        }

        std::vector<byte> recv_data(std::size_t size) const
        {
            std::vector<byte> data(size);
            recv(data.data(), data.size());
            return data;
        }

    private:
        void send(const byte* data, std::size_t size) const;
        void recv(byte* data, std::size_t size) const;

#ifdef PLATFORM_WIN
        addrinfo* connection_address;
        SOCKET connection_socket;

        static int connection_counter;
        static void static_init();
        static void static_destroy();
#endif // PLATFORM_WIN

#ifdef PLATFORM_UNIX
        sockaddr_in connection_address;
        int connection_socket;

        int try_connect(int sockfd, const sockaddr* addr, socklen_t addrlen);
#endif // PLATFORM_UNIX
    };
} // namespace nonstd

#include "windows/network_impl.hpp"
#include "unix/network_impl.hpp"
