﻿#pragma once
#include "../platform.hpp"

#ifdef PLATFORM_UNIX
#include "../processes.hpp"

#include <cstring>
#include <memory>
#include <stdexcept>
#include <vector>

#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

#include "../make_unique.hpp"
#include "../string_utils.hpp"

namespace nonstd
{
    inline process::process(const std::string& exe, const std::vector<std::string>& args, bool delayed_start/* = false*/)
        : exe(exe), args(args), pid(-1)
    {
        if (!delayed_start)
            start();
    }

    inline process::~process()
    {
        stop();
    }

    inline void process::start()
    {
        if (started())
            return;

        switch (pid = fork())
        {
        case -1:
            throw std::runtime_error("Couldn't create process"); // ToDo: better errors
        case 0:
            std::vector<std::unique_ptr<char[]>> argv_holder;
            std::vector<char*> argv;

            argv_holder.push_back(nonstd::strdup(exe));
            argv.push_back(argv_holder.back().get());
            for (const std::string& arg : args)
            {
                argv_holder.push_back(nonstd::strdup(arg));
                argv.push_back(argv_holder.back().get());
            }
            argv.push_back(nullptr);

            execvp(exe.c_str(), argv.data()); // ToDo: check error
            break;
        }
    }

    inline bool process::started() const
    {
        return pid > 0;
    }

    inline void process::stop() const
    {
        kill(pid, SIGKILL); // ToDo: error
    }

    inline void process::join() const
    {
        int ret;
        waitpid(pid, &ret, 0); // ToDo: error
    }
} // namespace nonstd
#endif // PLATFORM_UNIX
