package cz.cuni.mff.d3s;

import org.objectweb.asm.Label;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;

public class AddMethodExitHook extends MethodVisitor
{
    private String className;

    public AddMethodExitHook(MethodVisitor mv, String className)
    {
        super(Opcodes.ASM6, mv);
        this.className = className;
    }

    @Override
    public void visitInsn(int opcode)
    {
        if (opcode >= Opcodes.IRETURN && opcode <= Opcodes.RETURN)
        {
            mv.visitFieldInsn(Opcodes.GETSTATIC, "java/lang/Object", InstrumentationEngine.FieldHooksEnabledName, "Z");
            Label label = new Label();
            mv.visitJumpInsn(Opcodes.IFEQ, label);                

            switch (opcode)
            {
            case Opcodes.IRETURN:
                mv.visitInsn(Opcodes.DUP);
                mv.visitMethodInsn(Opcodes.INVOKESTATIC, "java/lang/Integer", "valueOf", "(I)Ljava/lang/Integer;", false);
                mv.visitMethodInsn(Opcodes.INVOKESTATIC, className, InstrumentationEngine.MethodExitHookName, "(Ljava/lang/Object;)V", false);
                break;
            case Opcodes.LRETURN:
                mv.visitInsn(Opcodes.DUP2);
                mv.visitMethodInsn(Opcodes.INVOKESTATIC, "java/lang/Long", "valueOf", "(J)Ljava/lang/Long;", false);
                mv.visitMethodInsn(Opcodes.INVOKESTATIC, className, InstrumentationEngine.MethodExitHookName, "(Ljava/lang/Object;)V", false);
                break;
            case Opcodes.FRETURN:
                mv.visitInsn(Opcodes.DUP);
                mv.visitMethodInsn(Opcodes.INVOKESTATIC, "java/lang/Float", "valueOf", "(F)Ljava/lang/Float;", false);
                mv.visitMethodInsn(Opcodes.INVOKESTATIC, className, InstrumentationEngine.MethodExitHookName, "(Ljava/lang/Object;)V", false);
                break;
            case Opcodes.DRETURN:
                mv.visitInsn(Opcodes.DUP2);
                mv.visitMethodInsn(Opcodes.INVOKESTATIC, "java/lang/Double", "valueOf", "(D)Ljava/lang/Double;", false);
                mv.visitMethodInsn(Opcodes.INVOKESTATIC, className, InstrumentationEngine.MethodExitHookName, "(Ljava/lang/Object;)V", false);
                break;
            case Opcodes.ARETURN:
                mv.visitInsn(Opcodes.DUP);
                mv.visitMethodInsn(Opcodes.INVOKESTATIC, className, InstrumentationEngine.MethodExitHookName, "(Ljava/lang/Object;)V", false);
                break;
            case Opcodes.RETURN:
                mv.visitInsn(Opcodes.ACONST_NULL);
                mv.visitMethodInsn(Opcodes.INVOKESTATIC, className, InstrumentationEngine.MethodExitHookName, "(Ljava/lang/Object;)V", false);
                break;
            }

            mv.visitLabel(label);
            // mv.visitFrame(Opcodes.??, 0, null, 0, null); // ignored, computed
        }
        mv.visitInsn(opcode);
    }
}
