package cz.cuni.mff.d3s;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;

public class BCIServer
{
    ServerSocket listener;
    Socket socket;

    public BCIServer(InetAddress addr, int port) throws BCIServerException
    {
        try
        {
            listener = new ServerSocket(port, 50, addr);
        }
        catch (IOException e) // ToDo
        {
            throw new BCIServerException("Can't initialize listener.", e);
        }
    }

    public void waitForConnection() throws BCIServerException
    {
        try
        {
            socket = listener.accept();
        }
        catch (IOException e) // ToDo
        {
            throw new BCIServerException("Can't accept connection.", e);
        }
    }

    public void close() throws IOException
    {
        if (socket != null)
            socket.close();
        if (listener != null)
            listener.close();
    }

    private final int REQUEST_MESSAGE_SIZE = 4;
    
    public boolean processRequest() throws BCIServerException
    {
        BCIServerRequest request = readRequest();
        sendResponse(request);
        return !request.exitRequested();
    }

    private BCIServerRequest readRequest() throws BCIServerException
    {
        try
        {
            DataInputStream iStream = new DataInputStream(socket.getInputStream());

            byte[] headerbuffer = new byte[REQUEST_MESSAGE_SIZE];
            iStream.readFully(headerbuffer);

            BCIServerRequest request = new BCIServerRequest(headerbuffer);
            if (request.awaitsData())
            {
                int datalength = iStream.readInt();

                byte[] dataBuffer = new byte[datalength]; // ToDo: Warning, length
                iStream.readFully(dataBuffer);
                request.setData(dataBuffer);
            }

            return request;
        }
        catch (IOException e)
        {
            throw new BCIServerException("Error in read.", e); // ToDo
        }
    }

    private boolean sendResponse(BCIServerRequest request) throws BCIServerException
    {
        try
        {
            request.sendResponse(new DataOutputStream(socket.getOutputStream()));
            return request.exitRequested();
        }
        catch (IOException e)
        {
            throw new BCIServerException("Error in write.", e);
        }
    }
}
