package cz.cuni.mff.d3s;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.Opcodes;

public class InstrumentationEngine
{
    private ClassReader classReader;
    private ClassWriter classWriter;

    public static final String MethodEntryHookName = "__MethodEntryHook__";
    public static final String MethodExitHookName = "__MethodExitHook__";
    public static final String FieldHooksEnabledName = "__JVMTIHooksEnabled__";

    public InstrumentationEngine(byte[] classData)
    {
        classReader = new ClassReader(classData);
        classWriter = new ClassWriter(classReader, ClassWriter.COMPUTE_FRAMES);
    }
    
    public byte[] instrumentObject()
    {
        ClassVisitor visitor = new AddFieldJVMTIHooksEnabled(classWriter);
        return performInstrumentation(visitor);
    }

    public byte[] instrumentGeneric()
    {
        if ((classReader.getAccess() & Opcodes.ACC_INTERFACE) == 0)
        {
            ClassVisitor visitor = new AddJVMTIMethodHooks(classWriter, classReader.getClassName());
            return performInstrumentation(visitor);
        }
        return new byte[0];
    }

    private byte[] performInstrumentation(ClassVisitor visitor)
    {
        classReader.accept(visitor, 0);
        return classWriter.toByteArray();
    }
}
