﻿#pragma once
#include <string>
#include "nonstd/processes.hpp"
#include "nonstd/network.hpp"

#include "bci_error.hpp"

namespace runtime_checker
{
    struct bci_server_configuration
    {
        bci_server_configuration(const std::string& jar_file, const std::string& address, int port)
            : jar_file(jar_file), address(address), port(port)
        {
        }

        std::string jar_file;
        std::string address;
        int port;
    };

    class instrumentation_server
    {
#ifdef _MSC_VER
        #pragma region protocol
#endif
            static constexpr const int MESSAGE_PROTOCOL_SIZE = 4;
            static constexpr const char* MESSAGE_SUCCESS = "SUCC";
            static constexpr const char* MESSAGE_INSTRUMENT_OBJECT = "INSO";
            static constexpr const char* MESSAGE_INSTRUMENT_GENERIC = "INSG";
            static constexpr const char* MESSAGE_INSTRUMENT_SUCCESS = "INSA";
            static constexpr const char* MESSAGE_INSTRUMENT_EMPTY = "INSE";
            static constexpr const char* MESSAGE_INSTRUMENT_FAILED = "INSR";
            static constexpr const char* MESSAGE_QUIT = "QUIT";
#ifdef _MSC_VER
        #pragma endregion protocol
#endif

    public:
        explicit instrumentation_server(const bci_server_configuration& config)
            : process("java", {"-jar", config.jar_file, config.address, std::to_string(config.port)}, true), connection(config.address, config.port, true)
        {
            process.start();
            connection.connect();
        }

        std::vector<nonstd::connection::byte> perform_instrumentation(const std::string& name, const unsigned char* data, jint data_len, bool object_instrumentation = false) const
        {
            connection.send_message(object_instrumentation ? MESSAGE_INSTRUMENT_OBJECT : MESSAGE_INSTRUMENT_GENERIC);

            connection.send_int32(data_len);
            connection.send_data(data, data_len);

            auto response = connection.recv_message(MESSAGE_PROTOCOL_SIZE);
            if (response == MESSAGE_INSTRUMENT_FAILED)
                throw bci_error("Server failed to instrument the class.");
            if (response == MESSAGE_INSTRUMENT_EMPTY)
                return {};
            if (response == MESSAGE_INSTRUMENT_SUCCESS)
            {
                auto size = connection.recv_int32();
                return connection.recv_data(size);
            }
            throw bci_error("Received invalid message '" + std::to_string(static_cast<int>(response.c_str()[0])) + "'.");
        }

        void join() const
        {
            connection.send_message(MESSAGE_QUIT);
            if (connection.recv_message(MESSAGE_PROTOCOL_SIZE) == MESSAGE_SUCCESS)
            {
                process.join();
                return;
            }
            process.stop();
        }

    private:
        nonstd::process process;
        nonstd::connection connection;
    };
} // namespace runtime_checker
