﻿#pragma once
#include "../platform.hpp"

#ifdef PLATFORM_WIN
#include "../processes.hpp"

#include <cstring>
#include <memory>
#include <stdexcept>
#include <string>
#include <vector>

#include <windows.h>

#include "../make_unique.hpp"

namespace nonstd
{
    inline process::process(const std::string& exe, const std::vector<std::string>& args, bool delayed_start/* = false*/)
        : exe(exe), args(args)
    {
        std::memset(&process_info, 0, sizeof process_info);
        std::memset(&startup_info, 0, sizeof startup_info);
        startup_info.cb = sizeof startup_info;

        if (!delayed_start)
            start();
    }

    inline process::~process()
    {
        stop();
        CloseHandle(process_info.hProcess);
        CloseHandle(process_info.hThread);
    }

    inline void process::start()
    {
        if (started())
            return;

        LPSTR lpFilePart;
        char filename[MAX_PATH];
        SearchPath(nullptr, exe.c_str(), ".exe", MAX_PATH, filename, &lpFilePart);

        std::string command_line_str = exe;
        for (const std::string& arg : args)
            command_line_str += " " + arg;

        std::unique_ptr<char[]> command_line = nonstd::make_unique<char[]>(command_line_str.length() + 1);
        std::copy(command_line_str.data(), command_line_str.data() + command_line_str.size() + 1, command_line.get());

        if (!CreateProcessA(filename, command_line.get(), nullptr, nullptr, false, 0, nullptr, nullptr, &startup_info, &process_info))
            throw std::runtime_error("Process could not be started"); // ToDo: error from CreateProcess
    }

    inline bool process::started() const
    {
        return process_info.hProcess != nullptr;
    }

    inline void process::stop() const
    {
        TerminateProcess(process_info.hProcess, 0);
        join();
    }

    inline void process::join() const
    {
        WaitForSingleObject(process_info.hProcess, INFINITE);
    }
} // namespace nonstd
#endif // PLATFORM_WIN
