package cz.cuni.mff.d3s;

import org.objectweb.asm.Label;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;

public class AddMethodEntryHook extends MethodVisitor
{
    private String className;
    private String methodName;

    private boolean added = false;

    public AddMethodEntryHook(MethodVisitor mv, String className, String methodName)
    {
        super(Opcodes.ASM6, mv);
        this.className = className;
        this.methodName = methodName;
    }

    private void addMethodEntryHook()
    {
        mv.visitFieldInsn(Opcodes.GETSTATIC, "java/lang/Object", InstrumentationEngine.FieldHooksEnabledName, "Z");
        Label label = new Label();
        mv.visitJumpInsn(Opcodes.IFEQ, label);
        mv.visitMethodInsn(Opcodes.INVOKESTATIC, className, InstrumentationEngine.MethodEntryHookName, "()V", false);
        mv.visitLabel(label);
        // mv.visitFrame(Opcodes.??, 0, null, 0, null); // ignored, computed
        added = true;
    }

    @Override
    public void visitCode()
    {
        if (!added && !"<init>".equals(methodName))
            addMethodEntryHook();
    }

    @Override
    public void visitMethodInsn(int opcode, String owner, String name, String desc, boolean ift)
    {
        if (added)
        {
            mv.visitMethodInsn(opcode, owner, name, desc, ift);
            return;
        }

        if ("<init>".equals(methodName))
        {
            mv.visitMethodInsn(opcode, owner, name, desc, ift);
            addMethodEntryHook();
        }
    }
}
