package cz.cuni.mff.d3s;

import java.io.DataOutputStream;
import java.io.IOException;

public class BCIServerRequest
{
    public static final String MESSAGE_SUCCESS = "SUCC";
    public static final String MESSAGE_INSTRUMENT_OBJECT = "INSO";
    public static final String MESSAGE_INSTRUMENT_GENERIC = "INSG";
    public static final String MESSAGE_INSTRUMENT_SUCCESS = "INSA";
    public static final String MESSAGE_INSTRUMENT_EMPTY = "INSE";
    public static final String MESSAGE_INSTRUMENT_FAILED = "INSR";
    public static final String MESSAGE_QUIT = "QUIT";

    private String header;
    private byte[] data;

    public BCIServerRequest(byte[] header)
    {
        this.header = new String(header);
    }

    public String getHeader()
    {
        return header;
    }

    public boolean awaitsData()
    {
        return header.equals(MESSAGE_INSTRUMENT_OBJECT) || header.equals(MESSAGE_INSTRUMENT_GENERIC);
    }

    public void setData(byte[] buffer)
    {
        data = buffer;
    }

    // ToDo: Rewrite to something better
    public void sendResponse(DataOutputStream stream) throws BCIServerException, IOException
    {
        switch (header)
        {
            case MESSAGE_QUIT:
                stream.write(MESSAGE_SUCCESS.getBytes());
                break;
            case MESSAGE_INSTRUMENT_OBJECT:
            case MESSAGE_INSTRUMENT_GENERIC:
                byte[] cdata = null;
                if (MESSAGE_INSTRUMENT_OBJECT.equals(header))
                    cdata = new InstrumentationEngine(data).instrumentObject();
                else
                    cdata = new InstrumentationEngine(data).instrumentGeneric();

                if (cdata == null)
                    stream.write(MESSAGE_INSTRUMENT_FAILED.getBytes());
                else if (cdata.length == 0)
                    stream.write(MESSAGE_INSTRUMENT_EMPTY.getBytes());
                else
                {
                    stream.write(MESSAGE_INSTRUMENT_SUCCESS.getBytes());
                    stream.writeInt(cdata.length);
                    stream.write(cdata);
                }
                break;
            default:
                throw new BCIServerException("Invalid message " + header);
        }
    }

    public boolean exitRequested()
    {
        return header.equals(MESSAGE_QUIT);
    }
}
